package it.polito.esame.bean;

/**
 * Bean for representing the dictionary information in the {@code parole}
 * data-set.
 * 
 * Read-write attributes: {@code idParola} and {@code nome}.
 * <p>
 * Equals and Hash Code based on the {@code nome} field.
 * 
 * @author Fulvio
 * 
 */
public class Parola {

	private int idParola;

	private String nome;

	public Parola(int idParola, String nome) {
		this.idParola = idParola;
		this.nome = nome;
	}

	public int getIdParola() {
		return idParola;
	}

	public void setIdParola(int idParola) {
		this.idParola = idParola;
	}

	public String getNome() {
		return nome;
	}

	public void setNome(String nome) {
		this.nome = nome;
	}

	/**
	 * Check whether the current Parola object is <em>similar</em> to the one
	 * passed as parameter {@code q}.
	 * 
	 * To be considered similar, the two words must be equal for any position,
	 * except at most one. Example: cane and cave.
	 * 
	 * Special case: identical words are similar, too. Special case: words of
	 * different length may not be similar.
	 * 
	 * @param q the Parola to be compared.
	 * @return {@code true} if the word is similar, {@code false} otherwise
	 */
	public boolean similarTo(Parola q) {

		String questa = this.getNome();
		String altra = q.getNome();

		if (questa.length() != altra.length())
			return false;

		int diff = 0;
		for (int i = 0; i < questa.length(); i++) {
			if (questa.charAt(i) != altra.charAt(i))
				diff++;
		}

		return diff <= 1;
	}

	public boolean linkedWith(Parola q) {

		String myEnd = this.getNome().substring(q.getNome().length()-2, q.getNome().length()) ;
		String otherStart = q.getNome().substring(0,2) ;
		
		//System.out.format("%s-%s  ", myEnd, otherStart) ;
		
		return myEnd.equals(otherStart) ;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((nome == null) ? 0 : nome.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Parola other = (Parola) obj;
		if (nome == null) {
			if (other.nome != null)
				return false;
		} else if (!nome.equals(other.nome))
			return false;
		return true;
	}


}
