package it.polito.esame.gui;

import it.polito.esame.ModelT1;
import it.polito.esame.bean.Parola;
import it.polito.esame.dao.ParoleDAO;

import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.layout.BorderPane;

public class ControllerT1 {

	private ModelT1 model;

	@FXML
	private ResourceBundle resources;

	@FXML
	private URL location;

	@FXML
	private Button btnCarica;

	@FXML
	private Button btnCerca;

	@FXML
	private BorderPane theRoot;

	@FXML
	private TextField txtLunghezza;

	@FXML
	private TextField txtParolaFinale;

	@FXML
	private TextField txtParolaIniziale;

	@FXML
	private TextArea txtResult;

	public ModelT1 getModel() {
		return model;
	}

	public void setModel(ModelT1 model) {
		this.model = model;
	}

	@FXML
	void initialize() {
		assert btnCarica != null : "fx:id=\"btnCarica\" was not injected: check your FXML file 'parole.fxml'.";
		assert btnCerca != null : "fx:id=\"btnCerca\" was not injected: check your FXML file 'parole.fxml'.";
		assert theRoot != null : "fx:id=\"theRoot\" was not injected: check your FXML file 'parole.fxml'.";
		assert txtLunghezza != null : "fx:id=\"txtLunghezza\" was not injected: check your FXML file 'parole.fxml'.";
		assert txtParolaFinale != null : "fx:id=\"txtParolaFinale\" was not injected: check your FXML file 'parole.fxml'.";
		assert txtParolaIniziale != null : "fx:id=\"txtParolaIniziale\" was not injected: check your FXML file 'parole.fxml'.";
		assert txtResult != null : "fx:id=\"txtResult\" was not injected: check your FXML file 'parole.fxml'.";

		btnCerca.setDisable(true);

	}

	@FXML
	void doCaricaParole(ActionEvent event) {

		String lunghezza = txtLunghezza.getText();

		int lun=0 ;
		try {
			lun = Integer.parseInt(lunghezza);
		} catch (NumberFormatException e) {
			txtResult.appendText("Illegal number\n") ;
			return ; // don't compute!
		}
		
		ParoleDAO dao = new ParoleDAO();
		List<Parola> lista = dao.getAllParolaWithLength(lun);
		model.setListaParole(lista, lun);
		
		txtResult.clear() ;

		txtResult.appendText(String.format("Caricate %d parole di lunghezza %d\n", lista.size(), lun));

		model.createGraph();
		txtResult.appendText(String.format("Creato grafo con %d vertici %d archi\n", model
				.getGraph().vertexSet().size(), model.getGraph().edgeSet()
				.size()));
		
		// cerca parola maggiormente connessa
		
		int maxConn = 0 ;
		Parola found = null ;
		for( Parola p: model.getListaParole() ) {
			int conn = model.getNumConn(p) ;
			if(conn>maxConn) {
				maxConn = conn ;
				found = p ;
			}
		}
		txtResult.appendText(String.format("La parola %s possiede %d connessioni\n", found.getNome(), maxConn)) ;
		
		btnCerca.setDisable(false) ;
	}

	/**
	 * Action handler for the "Cerca" button: 
	 * @param event
	 */
	@FXML
	void doCercaParole(ActionEvent event) {
		
		String iniziale = txtParolaIniziale.getText() ;
		String finale = txtParolaFinale.getText() ;
		
		if(iniziale.length()!=model.getLun() || finale.length()!=model.getLun()) {
			txtResult.appendText(String.format("Errore: le parole devono avere %d caratteri\n", model.getLun())) ;
			return ;
		}
		
		int pIniz = model.getListaParole().indexOf(new Parola(0, iniziale)) ;
		int pFinal = model.getListaParole().indexOf(new Parola(0, finale)) ;
		
		if(pIniz == -1 || pFinal == -1) {
			txtResult.appendText(String.format("Errore: le parole non esistono nel dizionario\n")) ;
			return ;
		}
		
		List<Parola> percorso = model.trovaPercorso( model.getListaParole().get(pIniz) , model.getListaParole().get(pFinal) ) ;
		
		
		if(percorso==null) {
			txtResult.appendText(String.format("Spiacente, nessun percorso valido\n")) ;
			return ;
		}
			
		txtResult.clear() ;

		for( Parola p: percorso ) {
			txtResult.appendText(p.getNome()+"\n") ;
		}
		
	}

}
